@@ -262,7 +262,7 @@ edit_permissions_response_cb (GObject *dialog,
 
 	e_ews_config_utils_run_in_thread_with_feedback (
 		GTK_WINDOW (dialog), dialog,
-		_("Writing folder permissions, please wait..."),
+		_("Writing folder permissions, please wait…"),
 		write_folder_permissions_thread,
 		write_folder_permissions_idle,
 		write_permissions, (GDestroyNotify) g_slist_free);
@@ -854,6 +854,7 @@ create_permissions_tree_view (GObject *dialog,
                               struct EEwsPermissionsDialogWidgets *widgets)
 {
 	GtkTreeView *tree_view;
+	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -861,8 +862,9 @@ create_permissions_tree_view (GObject *dialog,
 
 	g_return_val_if_fail (widgets != NULL, NULL);
 
-	tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (
-		GTK_TREE_MODEL (gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_BOOLEAN))));
+	model = GTK_TREE_MODEL (gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_BOOLEAN));
+	tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (model));
+	g_object_unref (model);
 
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "editable", FALSE, NULL);
@@ -924,7 +926,7 @@ e_ews_edit_folder_permissions (GtkWindow *parent,
 	widgets->folder_type = folder_type;
 
 	widgets->dialog = gtk_dialog_new_with_buttons (
-		_("Edit EWS folder permissions..."),
+		_("Edit EWS folder permissions…"),
 		parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -1308,7 +1310,7 @@ e_ews_edit_folder_permissions (GtkWindow *parent,
 
 	e_ews_config_utils_run_in_thread_with_feedback (
 		GTK_WINDOW (dialog), dialog,
-		_("Reading folder permissions, please wait..."),
+		_("Reading folder permissions, please wait…"),
 		read_folder_permissions_thread,
 		read_folder_permissions_idle,
 		found_permissions, folder_permissions_free_found_permissions);
